"""
https://docs.github.com/en/rest/repos/contents?apiVersion=2022-11-28
"""

import base64
from typing import Any, Dict, List

import orjson
import requests
from funsecret import read_secret

from fundrive.core import BaseDrive


class GithubDrive(BaseDrive):
    def __init__(self, *args, **kwargs):
        super(GithubDrive, self).__init__(*args, **kwargs)
        self.base_url = "https://api.github.com"
        self.repo_str = None
        self.headers = {}

    def login(self, repo_str, access_tokens=None, *args, **kwargs) -> bool:
        access_tokens = access_tokens or read_secret(
            cate1="fundrive",
            cate2="drives",
            cate3="fungithub",
            cate4="access_tokens",
        )
        self.headers = {"Authorization": f"Token {access_tokens}"}
        self.repo_str = repo_str
        return True

    def delete(self, git_path, *args, **kwargs) -> bool:
        requests.get(
            f"{self.base_url}/repos/{self.repo_str}/contents/{git_path}",
            headers=self.headers,
        )
        return True

    def get_dir_info(self, git_path, *args, **kwargs) -> Dict[str, Any]:
        data = self.get_dir_list(git_path)
        if len(data) > 0:
            return data[0]
        return {}

    def get_file_info(self, git_path, *args, **kwargs) -> Dict[str, Any]:
        res = requests.get(
            f"{self.base_url}/repos/{self.repo_str}/contents/{git_path}",
            headers=self.headers,
        )
        data = res.json()
        if len(data) == 0 or "name" not in data:
            return {}
        return {
            "name": data["name"],
            "path": data["path"],
            "size": data["size"],
            "sha": data["sha"],
            # "time": datetime.strptime(last_modified, "%a, %d %b %Y %H:%M:%S %Z").strftime("%Y-%m-%d %H:%M:%S"),
        }

    def get_file_list(
        self, git_path, recursive=False, *args, **kwargs
    ) -> List[Dict[str, Any]]:
        all_files = []
        res = requests.get(
            f"{self.base_url}/repos/{self.repo_str}/contents/{git_path}",
            headers=self.headers,
        )
        for data in res.json():
            if data["type"] != "dir":
                all_files.append(
                    {
                        "name": data["name"],
                        "path": data["path"],
                        "size": data["size"],
                        "sha": data["sha"],
                        # "time": datetime.strptime(last_modified, "%a, %d %b %Y %H:%M:%S %Z").strftime("%Y-%m-%d %H:%M:%S"),
                    }
                )
            elif recursive:
                all_files.extend(self.get_file_list(data["path"]))
        return all_files

    def get_dir_list(
        self, git_path, recursive=False, *args, **kwargs
    ) -> List[Dict[str, Any]]:
        all_files = []
        res = requests.get(
            f"{self.base_url}/repos/{self.repo_str}/contents/{git_path}",
            headers=self.headers,
        )
        for data in res.json():
            if data["type"] == "dir":
                all_files.append(
                    {
                        "name": data["name"],
                        "path": data["path"],
                        "size": data["size"],
                        # "time": datetime.strptime(last_modified, "%a, %d %b %Y %H:%M:%S %Z").strftime("%Y-%m-%d %H:%M:%S"),
                    }
                )
                if recursive:
                    all_files.extend(self.get_file_list(data["path"]))
        return all_files

    def upload_file(
        self,
        file_path="./cache",
        content=None,
        git_path=None,
        message="committing files",
        branch="master",
        overwrite=False,
        *args,
        **kwargs,
    ) -> bool:
        uri = f"{self.base_url}/repos/{self.repo_str}/contents/{git_path}"
        if content is None:
            content = open(file_path, "r").read()
        if not isinstance(content, str):
            content = orjson.dumps(content).decode("utf-8")

        data = {
            "message": message,
            "content": base64.b64encode(content.encode("utf-8")).decode(),
            "branch": branch,
        }
        exist_info = self.get_file_info(git_path)
        if "sha" in exist_info:
            data["sha"] = exist_info["sha"]

        response = requests.put(uri, headers=self.headers, json=data)
        if response.status_code not in (200, 201):
            print(response.status_code, response.text)
        return True



from funsecret import read_secret

from fundrive.drives import ZenodoDrive

drive = ZenodoDrive()
drive.login(access_token=read_secret("fundrive", "zenodo", "access_token"))
drive.check_token()

for file in drive.get_file_list("13293150"):
    print(file)
# 下载一个record下的所有文件
drive.download_file(record_id="13293150", local_dir="./resource/songs")
record_id = drive.mkdir()["metadata"]["prereserve_doi"]["recid"]

drive.upload_dir(local_path="./resource/songs", record_id=record_id)



import os
from typing import List

from fundrive.core import BaseDrive, DriveFile
from funget import simple_download
from funsecret import read_secret
from funutil import getLogger

from .client import ZenodoClient

logger = getLogger("fundrive")


class ZenodoDrive(BaseDrive):
    def __init__(self):
        super().__init__()
        self.access_token = None

        self.client = ZenodoClient(access_token=self.access_token)

    def check_token(self):
        if not isinstance(self.access_token, str) or not self.access_token:
            logger.error("Token need to be a string")
        return self.client.representation_create()[0]

    def login(self, access_token=None, *args, **kwargs) -> bool:
        self.access_token = access_token or read_secret(
            "fundrive", "zenodo", "access_token"
        )
        self.client.access_token = self.access_token
        return True

    def search_records(
        self,
        q=None,
        status=None,
        sort=None,
        page=None,
        size=None,
        all_versions=None,
        communities=None,
        type=None,
        subtype=None,
        bounds=None,
        custom=None,
        *args,
        **kwargs,
    ):
        """
        q
        string	optional	Search query (using Elasticsearch query string syntax - note that some characters have special meaning here, including /, which is also present in full DOIs).
        status
        string	optional	Filter result based on the deposit status (either draft or published)
        sort
        string	optional	Sort order (bestmatch or mostrecent). Prefix with minus to change form ascending to descending (e.g. -mostrecent).
        page
        integer	optional	Page number for pagination.
        size
        integer	optional	Number of results to return per page.
        all_versions
        integer/string	optional	Show (true or 1) or hide (false or 0) all versions of records.
        communities
        string	optional	Return records that are part of the specified communities. (Use of community identifier)
        type
        string	optional	Return records of the specified type. (Publication, Poster, Presentation…)
        subtype
        string	optional	Return records of the specified subtype. (Journal article, Preprint, Proposal…)
        bounds
        string	optional	Return records filtered by a geolocation bounding box. (Format bounds=143.37158,-38.99357,146.90918,-37.35269)
        custom
        string	optional	Return records containing the specified custom keywords. (Format custom=[field_name]:field_value)

        Returns:

        """
        return self.client.records_list(
            q=q,
            status=status,
            sort=sort,
            page=page,
            size=size,
            all_versions=all_versions,
            communities=communities,
            type=type,
            subtype=subtype,
            bounds=bounds,
            custom=custom,
            *args,
            **kwargs,
        )[1]

    def get_dir_list(self, q=None, *args, **kwargs) -> List[DriveFile]:
        response = self.search_records(q=q, *args, **kwargs)
        result = []
        for record in response["hits"]["hits"]:
            result.append(DriveFile(fid=record["id"], name=record["title"], **record))
        return result

    def get_file_info(
        self, fid=None, record_id=None, filepath=None, *args, **kwargs
    ) -> DriveFile:
        response = self.client.deposition_files_retrieve(
            record_id=record_id, file_id=filepath
        )[1]
        return DriveFile(fid=fid, name=response["key"], **response)

    def get_file_list(self, record_id, *args, **kwargs) -> List[DriveFile]:
        response = self.client.records_retrieve(record_id=record_id)[1]
        result = []
        print(response)
        for file in response["files"]:
            result.append(
                DriveFile(
                    fid=file["links"]["self"].rstrip("/content"),
                    path=file["key"],
                    name=file["key"],
                    size=file["size"],
                    url=file["links"]["self"],
                    id=file["id"],
                    checksum=file["checksum"],
                )
            )
        return result

    def download_file(
        self, record_id, local_dir, overwrite=False, *args, **kwargs
    ) -> bool:
        for file in self.get_file_list(record_id, *args, **kwargs):
            simple_download(url=file["url"], filepath=f"{local_dir}/{file['name']}")
        return True

    def download_dir(
        self, record_id, local_dir, recursion=True, overwrite=False, *args, **kwargs
    ) -> bool:
        return self.download_file(
            record_id, local_dir, overwrite=overwrite, *args, **kwargs
        )

    def mkdir(self, fid=None, name=None, return_if_exist=True, *args, **kwargs) -> dict:
        return self.client.representation_create()[1]

    def upload_dir(
        self,
        local_path,
        record_id=None,
        recursion=True,
        overwrite=False,
        check=True,
        *args,
        **kwargs,
    ) -> bool:
        if check:
            record_id = self.check_record(record_id=record_id)

        for file in os.listdir(local_path):
            file_path = os.path.join(local_path, file)
            if os.path.isfile(file_path):
                self.upload_file(
                    local_path=file_path,
                    record_id=record_id,
                    overwrite=overwrite,
                    check=False,
                )
            elif os.path.isdir(file_path) and recursion:
                self.upload_dir(
                    local_path=file_path,
                    record_id=record_id,
                    overwrite=overwrite,
                    check=False,
                )
        return True

    def upload_file(
        self,
        local_path,
        record_id=None,
        recursion=True,
        overwrite=False,
        check=True,
        *args,
        **kwargs,
    ) -> bool:
        if check:
            record_id = self.check_record(record_id, new_version=False)
        return self.client.deposition_files_upload(
            record_id=record_id, filepath=local_path
        )[0]

    def check_record(self, record_id=None, new_version=True, *args, **kwargs) -> int:
        if not self.client.records_retrieve(record_id):
            return True
        if new_version:
            response = self.client.deposition_actions_new_version(record_id=record_id)
            if response[0]:
                return response[1]["id"]
        raise Exception(f"No record found for {record_id}")

    def publish(
        self,
        record_id=None,
        title="My first upload",
        description="My first upload",
        names="fundrive",
        creators=None,
    ):
        self.client.representation_update(
            record_id=record_id,
            title=title,
            description=description,
            names=names,
            creators=creators,
        )
        return self.client.deposition_actions_publish(record_id=record_id)[1]



import json
import os

import requests
from funutil import getLogger
from funutil.cache import cache

logger = getLogger("fundrive")


class ZenodoClient(object):
    """
    https://developers.zenodo.org/
    """

    code_list = {
        200: {
            "code": 200,
            "name": "OK",
            "desc": "Request succeeded. Response included. Usually sent for GET/PUT/PATCH requests.",
        },
        201: {
            "code": 201,
            "name": "Created",
            "desc": "Request succeeded. Response included. Usually sent for POST requests.",
        },
        202: {
            "code": 202,
            "name": "Accepted",
            "desc": "Request succeeded. Response included. Usually sent for POST requests, where background processing is needed to fulfill the request.",
        },
        204: {
            "code": 204,
            "name": "No Content",
            "desc": "Request succeeded. No response included. Usually sent for DELETE requests.",
        },
        400: {
            "code": 400,
            "name": "Bad Request",
            "desc": "Request failed. Error response included.",
        },
        401: {
            "code": 401,
            "name": "Unauthorized",
            "desc": "Request failed, due to an invalid access token. Error response included.",
        },
        403: {
            "code": 403,
            "name": "Forbidden",
            "desc": "Request failed, due to missing authorization (e.g. deleting an already submitted upload or missing scopes for your access token). Error response included.",
        },
        404: {
            "code": 404,
            "name": "Not Found",
            "desc": "Request failed, due to the resource not being found. Error response included.",
        },
        405: {
            "code": 405,
            "name": "Method Not Allowed",
            "desc": "Request failed, due to unsupported HTTP method. Error response included.",
        },
        409: {
            "code": 409,
            "name": "Conflict",
            "desc": "Request failed, due to the current state of the resource (e.g. edit a deopsition which is not fully integrated). Error response included.",
        },
        415: {
            "code": 415,
            "name": "Unsupported Media Type",
            "desc": "Request failed, due to missing or invalid request header Content-Type. Error response included.",
        },
        429: {
            "code": 429,
            "name": "Too Many Requests",
            "desc": "Request failed, due to rate limiting. Error response included.",
        },
        500: {
            "code": 500,
            "name": "Internal Server Error",
            "desc": "Request failed, due to an internal server error. Error response NOT included. Don’t worry, Zenodo admins have been notified and will be dealing with the problem ASAP.",
        },
    }

    def __init__(self, access_token, sandbox=False):
        self.access_token = access_token
        self.base_url = (
            "https://sandbox.zenodo.org" if sandbox else "https://zenodo.org"
        )

    def __request(self, method, uri, params=None, *args, **kwargs):
        url = uri if uri.startswith("http") else f"{self.base_url}/{uri}"
        params = params or {"access_token": self.access_token}
        return requests.request(method, url, params=params, *args, **kwargs)

    def __check_status_code(self, r, status_code, prefix=""):
        if r.status_code != status_code:
            logger.error(
                f"{prefix} request error,status code: {r.status_code}:{self.code_list.get(r.status_code)}: {r.url}: {r.json().get('message')}"
            )
            return False
        return True

    def representation_list(
        self, q, status=None, sort=None, page=None, size=None, all_versions=None
    ) -> [bool, dict]:
        """
        q        string	optional	Search query (using Elasticsearch query string syntax - note that some characters have special meaning here, including /, which is also present in full DOIs).
        status        string	optional	Filter result based on deposit status (either draft or published)
        sort        string	optional	Sort order (bestmatch or mostrecent). Prefix with minus to change form ascending to descending (e.g. -mostrecent).
        page        integer	optional	Page number for pagination.
        size        integer	optional	Number of results to return per page.
        all_versions        integer/string	optional	Show (true or 1) or hide (false or 0) all versions of deposits.
        Returns:
        """
        uri = "/api/deposit/depositions"
        params = {
            "q": q,
            "status": status,
            "sort": sort,
            "page": page,
            "size": size,
            "all_versions": all_versions,
            "access_token": self.access_token,
        }
        r = self.__request("get", uri, params=params)
        return self.__check_status_code(r, 201), r.json()

    def representation_create(
        self,
    ) -> [bool, dict]:
        uri = "/api/deposit/depositions"
        data = {}
        headers = {"Content-Type": "application/json"}
        r = self.__request("post", uri, data=json.dumps(data), headers=headers)
        return self.__check_status_code(r, 201), r.json()

    def representation_retrieve(self, record_id):
        uri = f"/api/deposit/depositions/{record_id}"
        r = self.__request(
            "get",
            uri,
        )
        return self.__check_status_code(r, 200), r.json()

    def representation_update(
        self,
        record_id,
        title,
        description,
        names="farfarfun",
        creators=None,
    ) -> [bool, dict]:
        data = {
            "metadata": {
                "title": title,
                "upload_type": "poster",
                "description": description,
                "creators": creators
                or [
                    {"name": name, "affiliation": "farfarfun"}
                    for name in names.split(",")
                ],
            }
        }
        uri = f"/api/deposit/depositions/{record_id}"
        r = self.__request(
            "put",
            uri,
            params={"access_token": self.access_token},
            data=json.dumps(data),
            headers={"Content-Type": "application/json"},
        )
        return self.__check_status_code(r, 200), r.json()

    def representation_delete(self, record_id) -> [bool, dict]:
        uri = f"/api/deposit/depositions/{record_id}"
        r = self.__request(
            "delete",
            uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def deposition_files_list(self, record_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files"
        r = self.__request(
            "get",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    @cache
    def __get_deposition_id_by_record_id(
        self, record_id, *args, **kwargs
    ) -> [bool, dict]:
        status, retrieve = self.representation_retrieve(record_id=record_id)
        return retrieve["id"]

    @cache
    def __get_bucket_url_by_record_id(self, record_id, *args, **kwargs) -> [bool, dict]:
        status, retrieve = self.representation_retrieve(record_id=record_id)
        return retrieve["links"]["bucket"]

    def deposition_files_upload(
        self, record_id, filepath, filename=None, upload_type="old"
    ) -> [bool, dict]:
        bucket_url = self.__get_bucket_url_by_record_id(record_id=record_id)
        filename = filename or os.path.basename(filepath)
        if upload_type == "old":
            deposition_id = self.__get_deposition_id_by_record_id(record_id=record_id)

            r = self.__request(
                "post",
                f"/api/deposit/depositions/{deposition_id}/files",
                data={"name": filename},
                files={"file": open(filepath, "rb")},
            )
            if self.__check_status_code(r, 201, prefix=filename):
                logger.success(
                    f"{filepath} ID = {record_id} (DOI: 10.5281/zenodo.{record_id})"
                )
                return True, r.json()
            else:
                return False, r.json()
        else:
            with open(filepath, "rb") as fr:
                r = self.__request("put", uri=f"{bucket_url}/{filename}", data=fr)
                if self.__check_status_code(r, 201, prefix=filename):
                    logger.success(
                        f"{filepath} ID = {record_id} (DOI: 10.5281/zenodo.{record_id})"
                    )
                    return True, r.json()
                else:
                    return False, r.json()

    def deposition_files_create(
        self, record_id, filepath, filename=None
    ) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files"
        data = {"name": filename or os.path.basename(filepath)}
        files = {"file": open(filepath, "rb")}
        r = self.__request("post", uri=uri, data=data, files=files)
        return self.__check_status_code(r, 201), r.json()

    def deposition_files_sort(self, record_id, data) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files"
        headers = {"Content-Type": "application/json"}
        r = self.__request("put", uri=uri, data=json.dumps(data), headers=headers)
        return self.__check_status_code(r, 200), r.json()

    def deposition_files_retrieve(self, record_id, file_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files/{file_id}"
        r = self.__request(
            "get",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def deposition_files_update(self, record_id, filename=None) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files"
        data = {"name": filename}
        headers = {"Content-Type": "application/json"}
        r = self.__request("put", uri=uri, data=data, headers=headers)
        return self.__check_status_code(r, 201), r.json()

    def deposition_files_delete(self, record_id, file_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/files/{file_id}"
        r = self.__request(
            "delete",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def deposition_actions_publish(self, record_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/actions/publish"
        r = self.__request(
            "post",
            uri=uri,
        )
        return self.__check_status_code(r, 202), r.json()

    def deposition_actions_edit(self, record_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/actions/edit"
        r = self.__request(
            "post",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def deposition_actions_discard(self, record_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/actions/discard"
        r = self.__request(
            "post",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def deposition_actions_new_version(self, record_id) -> [bool, dict]:
        uri = f"api/deposit/depositions/{record_id}/actions/newversion"
        r = self.__request(
            "post",
            uri=uri,
        )
        return self.__check_status_code(r, 201), r.json()

    def records_list(
        self,
        q=None,
        status=None,
        sort=None,
        page=None,
        size=None,
        all_versions=None,
        communities=None,
        type=None,
        subtype=None,
        bounds=None,
        custom=None,
        *args,
        **kwargs,
    ) -> [bool, dict]:
        """
        q
        string	optional	Search query (using Elasticsearch query string syntax - note that some characters have special meaning here, including /, which is also present in full DOIs).
        status
        string	optional	Filter result based on the deposit status (either draft or published)
        sort
        string	optional	Sort order (best match or mostrecent). Prefix with minus to change form ascending to descending (e.g. -mostrecent).
        page
        integer	optional	Page number for pagination.
        size
        integer	optional	Number of results to return per page.
        all_versions
        integer/string	optional	Show (true or 1) or hide (false or 0) all versions of records.
        communities
        string	optional	Return records that are part of the specified communities. (Use of community identifier)
        type
        string	optional	Return records of the specified type. (Publication, Poster, Presentation…)
        subtype
        string	optional	Return records of the specified subtype. (Journal article, Preprint, Proposal…)
        bounds
        string	optional	Return records filtered by a geolocation bounding box. (Format bounds=143.37158,-38.99357,146.90918,-37.35269)
        custom
        string	optional	Return records containing the specified custom keywords. (Format custom=[field_name]:field_value)

        Returns:

        """
        payload = {
            "q": q,
            "status": status,
            "sort": sort,
            "page": page,
            "size": size,
            "all_versions": all_versions,
            "communities": communities,
            "type": type,
            "subtype": subtype,
            "bounds": bounds,
            "custom": custom,
        }
        r = requests.get(
            url=f"{self.base_url}/records",
            data=payload,
            headers={"Content-Type": "application/json"},
        )
        return self.__check_status_code(r, 200), r.json()

    def records_retrieve(self, record_id) -> [bool, dict]:
        uri = f"api/records/{record_id}"
        r = self.__request("get", uri=uri, params={})
        return self.__check_status_code(r, 200), r.json()
